@@ -14,7 +14,6 @@
 
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/builder/basic/kvp.hpp>
-
 #include <mongocxx/client.hpp>
 #include <mongocxx/instance.hpp>
 #include <mongocxx/uri.hpp>
@@ -32,30 +31,14 @@ int main(int, char**) {
     auto db = conn["test"];
 
     // Remove all documents that match a condition.
-    {
-        // @begin: cpp-remove-matching-documents
-        db["restaurants"].delete_many(make_document(kvp("borough", "Manhattan")));
-        // @end: cpp-remove-matching-documents
-    }
+    { db["restaurants"].delete_many(make_document(kvp("borough", "Manhattan"))); }
 
     // Remove one document that matches a condition.
-    {
-        // @begin: cpp-remove-justone
-        db["restaurants"].delete_one(make_document(kvp("borough", "Queens")));
-        // @end: cpp-remove-justone
-    }
+    { db["restaurants"].delete_one(make_document(kvp("borough", "Queens"))); }
 
     // Remove all documents in a collection.
-    {
-        // @begin: cpp-remove-all-documents
-        db["restaurants"].delete_many({});
-        // @end: cpp-remove-all-documents
-    }
+    { db["restaurants"].delete_many({}); }
 
     // Drop a collection.
-    {
-        // @begin: cpp-drop-collection
-        db["restaurants"].drop();
-        // @end: cpp-drop-collection
-    }
+    { db["restaurants"].drop(); }
 }
