@@ -24,6 +24,12 @@
 namespace bsoncxx {
 BSONCXX_INLINE_NAMESPACE_BEGIN
 
+namespace types {
+namespace bson_value {
+class view;
+}  // namespace bson_value
+}  // namespace types
+
 namespace array {
 
 ///
@@ -41,37 +47,38 @@ class BSONCXX_API element : private document::element {
 
     using document::element::type;
 
-    using document::element::get_double;
-    using document::element::get_utf8;
-    using document::element::get_document;
     using document::element::get_array;
     using document::element::get_binary;
-    using document::element::get_undefined;
-    using document::element::get_oid;
     using document::element::get_bool;
-    using document::element::get_date;
-    using document::element::get_null;
-    using document::element::get_regex;
-    using document::element::get_dbpointer;
     using document::element::get_code;
-    using document::element::get_symbol;
     using document::element::get_codewscope;
-    using document::element::get_int32;
-    using document::element::get_timestamp;
-    using document::element::get_int64;
+    using document::element::get_date;
+    using document::element::get_dbpointer;
     using document::element::get_decimal128;
-    using document::element::get_minkey;
+    using document::element::get_document;
+    using document::element::get_double;
+    using document::element::get_int32;
+    using document::element::get_int64;
     using document::element::get_maxkey;
+    using document::element::get_minkey;
+    using document::element::get_null;
+    using document::element::get_oid;
+    using document::element::get_regex;
+    using document::element::get_string;
+    using document::element::get_symbol;
+    using document::element::get_timestamp;
+    using document::element::get_undefined;
+    using document::element::get_utf8;
 
     using document::element::get_value;
 
     using document::element::operator[];
 
-    using document::element::raw;
+    using document::element::key;
+    using document::element::keylen;
     using document::element::length;
     using document::element::offset;
-    using document::element::keylen;
-    using document::element::key;
+    using document::element::raw;
 
    private:
     friend class view;
@@ -82,6 +89,36 @@ class BSONCXX_API element : private document::element {
                                      std::uint32_t keylen);
 };
 
+///
+/// @{
+///
+/// Convenience methods to compare for equality against a bson_value.
+///
+/// Returns true if this element contains a bson_value that matches.
+///
+/// @relates element
+///
+BSONCXX_API bool BSONCXX_CALL operator==(const element& elem, const types::bson_value::view& v);
+BSONCXX_API bool BSONCXX_CALL operator==(const types::bson_value::view& v, const element& elem);
+///
+/// @}
+///
+
+///
+/// @{
+///
+/// Convenience methods to compare for equality against a bson_value.
+///
+/// Returns false if this element contains a bson_value that matches.
+///
+/// @relates element
+///
+BSONCXX_API bool BSONCXX_CALL operator!=(const element& elem, const types::bson_value::view& v);
+BSONCXX_API bool BSONCXX_CALL operator!=(const types::bson_value::view& v, const element& elem);
+///
+/// @}
+///
+
 }  // namespace array
 
 BSONCXX_INLINE_NAMESPACE_END
