@@ -65,6 +65,7 @@ MONGOCXX_INLINE_NAMESPACE_BEGIN
 
 class client;
 class database;
+class client_encryption;
 
 ///
 /// Class representing server side document groupings within a MongoDB database.
@@ -699,7 +700,8 @@ class MONGOCXX_API collection {
     /// @note
     ///   Write concern supported only for MongoDB 3.4+.
     ///
-    void drop(const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern = {});
+    void drop(const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern = {},
+              bsoncxx::document::view_or_value collection_options = {});
 
     ///
     /// Drops this collection and all its contained documents from the database.
@@ -720,7 +722,8 @@ class MONGOCXX_API collection {
     ///   Write concern supported only for MongoDB 3.4+.
     ///
     void drop(const client_session& session,
-              const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern = {});
+              const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern = {},
+              bsoncxx::document::view_or_value collection_options = {});
 
     ///
     /// @}
@@ -1846,8 +1849,9 @@ class MONGOCXX_API collection {
     ///
 
    private:
-    friend class bulk_write;
-    friend class database;
+    friend mongocxx::bulk_write;
+    friend mongocxx::database;
+    friend mongocxx::client_encryption;
 
     MONGOCXX_PRIVATE collection(const database& database,
                                 bsoncxx::string::view_or_value collection_name);
@@ -1889,7 +1893,8 @@ class MONGOCXX_API collection {
 
     MONGOCXX_PRIVATE void _drop(
         const client_session* session,
-        const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern);
+        const bsoncxx::stdx::optional<mongocxx::write_concern>& write_concern,
+        bsoncxx::document::view_or_value collection_options);
 
     MONGOCXX_PRIVATE cursor _find(const client_session* session,
                                   bsoncxx::document::view_or_value filter,
