@@ -28,7 +28,7 @@ namespace mongocxx {
 MONGOCXX_INLINE_NAMESPACE_BEGIN
 namespace options {
 
-auto_encryption::auto_encryption() noexcept : _bypass(false) {}
+auto_encryption::auto_encryption() noexcept : _bypass(false), _bypass_query_analysis(false) {}
 
 auto_encryption& auto_encryption::key_vault_client(mongocxx::client* client) {
     _key_vault_client = client;
@@ -84,6 +84,17 @@ const stdx::optional<bsoncxx::document::view_or_value>& auto_encryption::schema_
     return _schema_map;
 }
 
+auto_encryption& auto_encryption::encrypted_fields_map(
+    bsoncxx::document::view_or_value encrypted_fields_map) {
+    _encrypted_fields_map = std::move(encrypted_fields_map);
+    return *this;
+}
+
+const stdx::optional<bsoncxx::document::view_or_value>& auto_encryption::encrypted_fields_map()
+    const {
+    return _encrypted_fields_map;
+}
+
 auto_encryption& auto_encryption::bypass_auto_encryption(bool should_bypass) {
     _bypass = should_bypass;
     return *this;
@@ -93,6 +104,15 @@ bool auto_encryption::bypass_auto_encryption() const {
     return _bypass;
 }
 
+auto_encryption& auto_encryption::bypass_query_analysis(bool should_bypass) {
+    _bypass_query_analysis = should_bypass;
+    return *this;
+}
+
+bool auto_encryption::bypass_query_analysis() const {
+    return _bypass_query_analysis;
+}
+
 auto_encryption& auto_encryption::extra_options(bsoncxx::document::view_or_value extra) {
     _extra_options = std::move(extra);
     return *this;
@@ -147,10 +167,20 @@ void* auto_encryption::convert() const {
         libmongoc::auto_encryption_opts_set_schema_map(mongoc_auto_encrypt_opts, schema_map.bson());
     }
 
+    if (_encrypted_fields_map) {
+        scoped_bson_t encrypted_fields_map{*_encrypted_fields_map};
+        libmongoc::auto_encryption_opts_set_encrypted_fields_map(mongoc_auto_encrypt_opts,
+                                                                 encrypted_fields_map.bson());
+    }
+
     if (_bypass) {
         libmongoc::auto_encryption_opts_set_bypass_auto_encryption(mongoc_auto_encrypt_opts, true);
     }
 
+    if (_bypass_query_analysis) {
+        libmongoc::auto_encryption_opts_set_bypass_query_analysis(mongoc_auto_encrypt_opts, true);
+    }
+
     if (_extra_options) {
         scoped_bson_t extra{*_extra_options};
         libmongoc::auto_encryption_opts_set_extra(mongoc_auto_encrypt_opts, extra.bson());
