@@ -237,6 +237,33 @@ class MONGOCXX_API auto_encryption {
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& schema_map() const;
 
+    ///
+    /// Sets the local encrypted fields map.
+    ///
+    /// Supplying an encryptedFieldsMap provides more security than relying on
+    /// an encryptedFields obtained from the server. It protects against a
+    /// malicious server advertising a false encryptedFields.
+    ///
+    /// @param encrypted_fields_map
+    ///   The mapping of which fields to encrypt.
+    ///
+    /// @see https://docs.mongodb.com/manual/core/security-client-side-encryption/
+    ///
+    /// @return
+    ///   A reference to this object to facilitate method chaining.
+    ///
+    /// @see https://docs.mongodb.com/manual/core/security-client-side-encryption/
+    ///
+    auto_encryption& encrypted_fields_map(bsoncxx::document::view_or_value encrypted_fields_map);
+
+    ///
+    /// Get encrypted fields map
+    ///
+    /// @return
+    ///   An optional document containing the encrypted fields map
+    ///
+    const stdx::optional<bsoncxx::document::view_or_value>& encrypted_fields_map() const;
+
     ///
     /// Automatic encryption is disabled when the 'bypassAutoEncryption'
     /// option is true. Default is 'false,' so auto encryption is enabled.
@@ -259,6 +286,28 @@ class MONGOCXX_API auto_encryption {
     ///
     bool bypass_auto_encryption() const;
 
+    ///
+    /// Query analysis is disabled when the 'bypassQueryAnalysis'
+    /// option is true. Default is 'false' (i.e. query analysis is enabled).
+    ///
+    /// @param should_bypass
+    ///   Whether or not to bypass query analysis.
+    ///
+    /// @return
+    ///   A reference to this object to facilitate method chaining.
+    ///
+    /// @see https://docs.mongodb.com/manual/core/security-client-side-encryption/
+    ///
+    auto_encryption& bypass_query_analysis(bool should_bypass);
+
+    ///
+    /// Gets a boolean specifying whether or not query analysis is bypassed.
+    ///
+    /// @return
+    ///   A boolean specifying whether query analysis is bypassed.
+    ///
+    bool bypass_query_analysis() const;
+
     ///
     /// Set extra options related to the mongocryptd process. This options
     /// document may include the following fields:
@@ -273,6 +322,33 @@ class MONGOCXX_API auto_encryption {
     /// - mongocryptdSpawnArgs: array[strings], options passed to mongocryptd
     ///   when spawing. Defaults to ["--idleShutdownTimeoutSecs=60"].
     ///
+    /// - cryptSharedLibPath - Set a filepath string referring to a crypt_shared library file. Unset
+    ///   by default. If not set (the default), libmongocrypt will attempt to load crypt_shared
+    ///   using the host system’s default dynamic-library-search system.
+    ///
+    ///   If set, the given path should identify the crypt_shared dynamic library file itself, not
+    ///   the directory that contains it.
+    ///
+    ///   If the given path is a relative path and the first path component is $ORIGIN, the $ORIGIN
+    ///   component will be replaced with the absolute path to the directory containing the
+    ///   libmongocrypt library in use by the application.
+    ///
+    ///   Note No other RPATH/RUNPATH-style substitutions are available.
+    ///   If the given path is a relative path, the path will be resolved relative to the working
+    ///   directory of the operating system process.
+    ///
+    ///   If this option is set and libmongocrypt fails to load crypt_shared from the given
+    ///   filepath, libmongocrypt will fail to initialize and will not attempt to search for
+    ///   crypt_shared in any other locations.
+    ///
+    /// - cryptSharedLibRequired - If set to true, and libmongocrypt fails to load a crypt_shared
+    ///   library, initialization of auto-encryption will fail immediately and will not attempt to
+    ///   spawn mongocryptd.
+    ///
+    ///   If set to false (the default), cryptSharedLibPath is not set, and libmongocrypt fails to
+    ///   load crypt_shared, then libmongocrypt will proceed without crypt_shared and fall back to
+    ///   using mongocryptd.
+    ///
     /// @param extra
     ///   The extra options to set.
     ///
@@ -298,12 +374,14 @@ class MONGOCXX_API auto_encryption {
     MONGOCXX_PRIVATE void* convert() const;
 
     bool _bypass;
+    bool _bypass_query_analysis;
     stdx::optional<mongocxx::client*> _key_vault_client;
     stdx::optional<mongocxx::pool*> _key_vault_pool;
     stdx::optional<ns_pair> _key_vault_namespace;
     stdx::optional<bsoncxx::document::view_or_value> _kms_providers;
     stdx::optional<bsoncxx::document::view_or_value> _tls_opts;
     stdx::optional<bsoncxx::document::view_or_value> _schema_map;
+    stdx::optional<bsoncxx::document::view_or_value> _encrypted_fields_map;
     stdx::optional<bsoncxx::document::view_or_value> _extra_options;
 };
 
