@@ -12,9 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <sstream>
-
 #include <helpers.hpp>
+#include <sstream>
 
 #include <bsoncxx/private/helpers.hh>
 #include <bsoncxx/stdx/make_unique.hpp>
@@ -29,9 +28,9 @@
 
 namespace {
 using bsoncxx::from_json;
-using bsoncxx::document::value;
 using bsoncxx::builder::basic::kvp;
 using bsoncxx::builder::basic::make_document;
+using bsoncxx::document::value;
 using bsoncxx::types::b_timestamp;
 
 using namespace mongocxx;
@@ -40,26 +39,116 @@ using test_util::server_has_sessions;
 TEST_CASE("session options", "[session]") {
     instance::current();
 
-    client c{uri{}};
+    client c{uri{}, test_util::add_test_server_api()};
 
     if (!server_has_sessions(c)) {
         return;
     }
 
     SECTION("default") {
-        auto s = c.start_session();
+        // Make sure the defaults don't cause a client exception:
+        options::client_session opts;
 
-        REQUIRE(s.options().causal_consistency());
+        REQUIRE_NOTHROW(c.start_session(opts));
     }
 
-    SECTION("set causal consistency") {
+    SECTION("default-- check values") {
+        auto s = c.start_session();
+
+        CHECK(s.options().causal_consistency());
+        CHECK_FALSE(s.options().snapshot());
+    }
+
+    SECTION("default causal consistency") {
         options::client_session opts;
+
+        // If internally un-set, we "see" the default value of true on read:
         REQUIRE(opts.causal_consistency());
+    }
+
+    SECTION("manually set causal consistency") {
+        options::client_session opts;
+
         opts.causal_consistency(false);
-        REQUIRE(!opts.causal_consistency());
+        REQUIRE_FALSE(opts.causal_consistency());
 
         auto s = c.start_session(opts);
-        REQUIRE(!s.options().causal_consistency());
+
+        // Setting for causal consistency should now be present, but false:
+        REQUIRE_FALSE(s.options().causal_consistency());
+    }
+
+    SECTION("manually set snapshot consistency") {
+        options::client_session opts;
+
+        REQUIRE_FALSE(opts.snapshot());
+        opts.causal_consistency(false);
+        opts.snapshot(true);
+
+        REQUIRE(opts.snapshot());
+        REQUIRE_FALSE(opts.causal_consistency());
+
+        auto s = c.start_session(opts);
+
+        // Nothing from the preconditions should have changed (because we
+        // specified values above; i.e. there have been no side-effects):
+        REQUIRE(opts.snapshot());
+        REQUIRE_FALSE(opts.causal_consistency());
+
+        // ...and the returned options have been set:
+        REQUIRE(s.options().snapshot());
+        REQUIRE_FALSE(s.options().causal_consistency());
+    }
+
+    SECTION("causal and snapshot consistency are mutually exclusive") {
+        // Trying to set both casusal and snapshot consistency should result
+        // in an error when starting the session:
+        options::client_session opts;
+
+        opts.snapshot(true);
+        opts.causal_consistency(true);
+
+        REQUIRE(opts.causal_consistency());
+        REQUIRE(opts.snapshot());
+
+        REQUIRE_THROWS_AS(c.start_session(opts), mongocxx::exception);
+    }
+
+    SECTION("invalid combinations of causal and snapshot consistency should fail") {
+        // Essentially, we can expect any number of combinations of un-set (i.e. does
+        // not contain a value), true, or false to be possible; but in the end any result
+        // leading to a final combination of [present, true, true] should result in an
+        // exception.
+
+        options::client_session opts;
+
+        // Unfortunately, stdx::nullopt doesn't always play well with others, so we'll use
+        // an enumeration:
+        enum struct optional_state { empty, no, yes };
+
+        auto causal_consistenty_opt = GENERATE(Catch::Generators::as<optional_state>{},
+                                               optional_state::empty,
+                                               optional_state::no,
+                                               optional_state::yes);
+        auto snapshot_consistenty_opt = GENERATE(Catch::Generators::as<optional_state>{},
+                                                 optional_state::empty,
+                                                 optional_state::no,
+                                                 optional_state::yes);
+
+        // Only actually set a value if we generate a non-empty setting:
+        if (optional_state::empty != causal_consistenty_opt)
+            opts.causal_consistency(optional_state::yes == causal_consistenty_opt);
+
+        if (optional_state::empty != snapshot_consistenty_opt)
+            opts.snapshot(optional_state::yes == snapshot_consistenty_opt);
+
+        // We should always expect an error if both are enabled:
+        if (optional_state::yes == causal_consistenty_opt &&
+            optional_state::yes == snapshot_consistenty_opt) {
+            REQUIRE_THROWS_AS(c.start_session(opts), mongocxx::exception);
+        } else {
+            CHECK_NOTHROW(c.start_session(opts));
+        }
     }
 }
 
@@ -77,7 +166,7 @@ TEST_CASE("start_session failure", "[session]") {
         })
         .forever();
 
-    client c{uri{}};
+    client c{uri{}, test_util::add_test_server_api()};
 
     REQUIRE_THROWS_MATCHES(
         c.start_session(), mongocxx::exception, mongocxx_exception_matcher{"foo"});
@@ -88,7 +177,7 @@ TEST_CASE("session", "[session]") {
 
     instance::current();
 
-    client c{uri{}};
+    client c{uri{}, test_util::add_test_server_api()};
 
     if (!server_has_sessions(c)) {
         return;
@@ -147,7 +236,7 @@ TEST_CASE("session", "[session]") {
 
         // "Session argument is for the right client" test from Driver Sessions Spec.
         // Passing a session from client "c" should fail with client "c2" and related objects.
-        client c2{uri{}};
+        client c2{uri{}, test_util::add_test_server_api()};
         auto db2 = c2["db"];
         auto collection2 = db2["collection"];
 
@@ -228,10 +317,11 @@ class session_test {
         apm_opts.on_command_started([&](const events::command_started_event& event) {
             std::string command_name{event.command_name().data()};
 
-            // Ignore auth commands like "saslStart", and handshakes with "isMaster".
+            // Ignore auth commands like "saslStart", and handshakes with "hello" (and the legacy
+            // "hello" command).
             std::string sasl{"sasl"};
             if (event.command_name().substr(0, sasl.size()).compare(sasl) == 0 ||
-                command_name.compare("isMaster") == 0) {
+                command_name.compare("hello") == 0 || command_name.compare("isMaster") == 0) {
                 return;
             }
 
@@ -239,7 +329,7 @@ class session_test {
         });
 
         client_opts.apm_opts(apm_opts);
-        client = mongocxx::client(uri{}, client_opts);
+        client = mongocxx::client(uri{}, test_util::add_test_server_api(client_opts));
     }
 
     void test_method_with_session(const std::function<void(bool)>& f, const client_session& s) {
