@@ -28,8 +28,8 @@
 #include <mongocxx/test_util/client_helpers.hh>
 
 namespace {
-using bsoncxx::builder::basic::make_document;
 using bsoncxx::builder::basic::kvp;
+using bsoncxx::builder::basic::make_document;
 using namespace mongocxx;
 
 bool test_commands_enabled(const client& conn) {
@@ -71,7 +71,7 @@ void disable_fail_point(const client& conn) {
 TEST_CASE("create_one", "[index_view]") {
     instance::current();
 
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     database db = mongodb_client["index_view_create_one"];
 
     SECTION("works with document and options") {
@@ -217,7 +217,7 @@ TEST_CASE("create_one", "[index_view]") {
 TEST_CASE("create_many", "[index_view]") {
     instance::current();
 
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     database db = mongodb_client["index_view_create_many"];
 
     std::vector<index_model> models{index_model(make_document(kvp("a", 1))),
@@ -256,7 +256,7 @@ TEST_CASE("create_many", "[index_view]") {
         std::int8_t found = 0;
 
         for (auto&& index : indexes.list()) {
-            auto name = index["name"].get_utf8();
+            auto name = index["name"].get_string();
 
             for (auto expected : expected_names) {
                 if (stdx::string_view(expected) == name.value) {
@@ -271,7 +271,7 @@ TEST_CASE("create_many", "[index_view]") {
 TEST_CASE("drop_one", "[index_view]") {
     instance::current();
 
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     database db = mongodb_client["index_view_drop_one"];
 
     SECTION("drops index by name") {
@@ -366,7 +366,7 @@ TEST_CASE("drop_one", "[index_view]") {
 TEST_CASE("drop_all", "[index_view]") {
     instance::current();
 
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     database db = mongodb_client["index_view_drop_all"];
 
     std::vector<index_model> models{index_model{make_document(kvp("a", 1))},
@@ -422,7 +422,7 @@ TEST_CASE("drop_all", "[index_view]") {
 TEST_CASE("index creation and deletion with different collation") {
     instance::current();
 
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
 
     if (test_util::get_max_wire_version(mongodb_client) >= 5) {
         database db = mongodb_client["index_view_collation"];
@@ -458,10 +458,11 @@ TEST_CASE("index creation and deletion with different collation") {
         ++index_it;
         bsoncxx::document::view index = *index_it;
 
-        REQUIRE(bsoncxx::string::to_string(index["name"].get_utf8().value) == "custom_index_name");
+        REQUIRE(bsoncxx::string::to_string(index["name"].get_string().value) ==
+                "custom_index_name");
 
         coll.drop();
         db.drop();
     }
 }
-}
+}  // namespace
