@@ -114,6 +114,22 @@ options::find make_find_options(document::view arguments) {
     return options;
 }
 
+document::value bulk_write_result_to_document(const mongocxx::result::bulk_write& result) {
+    builder::basic::document upserted_ids_builder;
+    for (auto&& index_and_id : result.upserted_ids()) {
+        upserted_ids_builder.append(
+            kvp(std::to_string(index_and_id.first), index_and_id.second.get_int32().value));
+    }
+
+    return make_document(kvp("matchedCount", result.matched_count()),
+                         kvp("modifiedCount", result.matched_count()),
+                         kvp("deletedCount", result.deleted_count()),
+                         kvp("insertedCount", result.inserted_count()),
+                         kvp("upsertedCount", result.upserted_count()),
+                         kvp("insertedIds", make_document()),
+                         kvp("upsertedIds", upserted_ids_builder.extract()));
+}
+
 document::value find(collection& coll, client_session* session, document::view operation) {
     document::view arguments = operation["arguments"].get_document().value;
     document::value empty_filter = builder::basic::make_document();
@@ -170,11 +186,68 @@ document::value list_collections(entity::map& map,
     return result.extract();
 }
 
+document::value list_collection_names(entity::map& map,
+                                      client_session* session,
+                                      const std::string& object,
+                                      document::view op) {
+    const auto arguments = op["arguments"];
+    const auto empty_doc = make_document();
+
+    document::view filter;
+    if (const auto f = arguments["filter"]) {
+        filter = f.get_document().value;
+    } else {
+        filter = empty_doc;
+    }
+
+    auto cursor = session ? map.get_database(object).list_collection_names(*session, filter)
+                          : map.get_database(object).list_collection_names(filter);
+
+    builder::basic::document result;
+    result.append(builder::basic::kvp("result", [&cursor](builder::basic::sub_array array) {
+        for (auto&& document : cursor) {
+            array.append(document);
+        }
+    }));
+    return result.extract();
+}
+
 document::value list_databases(entity::map& map,
                                client_session* session,
-                               const std::string& object) {
-    auto cursor = session ? map.get_client(object).list_databases(*session)
-                          : map.get_client(object).list_databases();
+                               const std::string& object,
+                               document::view op) {
+    const auto arguments = op["arguments"];
+    const auto empty_doc = make_document();
+    const auto arguments_view = arguments ? arguments.get_document().value : empty_doc.view();
+
+    auto cursor = session ? map.get_client(object).list_databases(*session, arguments_view)
+                          : map.get_client(object).list_databases(arguments_view);
+
+    builder::basic::document result;
+    result.append(builder::basic::kvp("result", [&cursor](builder::basic::sub_array array) {
+        for (auto&& document : cursor) {
+            array.append(document);
+        }
+    }));
+    return result.extract();
+}
+
+document::value list_database_names(entity::map& map,
+                                    client_session* session,
+                                    const std::string& object,
+                                    document::view op) {
+    const auto arguments = op["arguments"];
+    const auto empty_doc = make_document();
+
+    document::view filter;
+    if (const auto f = arguments["filter"]) {
+        filter = f.get_document().value;
+    } else {
+        filter = empty_doc;
+    }
+
+    auto cursor = session ? map.get_client(object).list_database_names(*session, filter)
+                          : map.get_client(object).list_database_names(filter);
 
     builder::basic::document result;
     result.append(builder::basic::kvp("result", [&cursor](builder::basic::sub_array array) {
@@ -551,6 +624,10 @@ document::value aggregate(Entity& entity, client_session* session, document::vie
         options.comment(comment.get_value());
     }
 
+    if (const auto allow_disk_use = arguments["allowDiskUse"]) {
+        options.allow_disk_use(allow_disk_use.get_bool());
+    }
+
     stdx::optional<cursor> result_cursor;
 
     if (session) {
@@ -714,9 +791,12 @@ document::value fail_point(entity::map& map, spec::apm_checker& apm, document::v
     auto client_name = string::to_string(args["client"].get_string().value);
     auto& client = map.get_client(client_name);
 
+    // The test runner MUST also ensure that the `configureFailPoint` command is excluded from the
+    // list of observed command monitoring events for this client (if applicable).
+    apm.set_ignore_command_monitoring_event("configureFailPoint");
+
     client["admin"].run_command(args["failPoint"].get_document().value);
 
-    apm.set_ignore_command_monitoring_event("configureFailPoint");
     return make_document(kvp("uri", client.uri().to_string()),
                          kvp("failPoint", args["failPoint"]["configureFailPoint"].get_string()));
 }
@@ -1524,6 +1604,170 @@ document::value distinct(collection& coll, client_session* session, document::vi
     return result.extract();
 }
 
+document::value rewrap_many_datakey(entity::map& map,
+                                    const std::string& object,
+                                    document::view operation) {
+    auto filter = operation["arguments"]["filter"].get_document();
+
+    options::rewrap_many_datakey rewrap_opts;
+    std::string provider;
+    if (operation["arguments"]["opts"]["provider"]) {
+        provider = std::string(operation["arguments"]["opts"]["provider"].get_string().value);
+    }
+    rewrap_opts.provider(provider);
+
+    if (operation["arguments"]["opts"]["masterKey"]) {
+        auto master_key = operation["arguments"]["opts"]["masterKey"].get_document().view();
+        rewrap_opts.master_key(master_key);
+    }
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.rewrap_many_datakey(filter.view(), rewrap_opts);
+
+    auto maybe_bulk_write = result.result();
+    if (maybe_bulk_write) {
+        auto bulk_write_result = maybe_bulk_write.value();
+        auto doc = make_document(
+            kvp("result",
+                make_document(
+                    kvp("bulkWriteResult", bulk_write_result_to_document(bulk_write_result)))));
+        return doc;
+    } else {
+        auto doc =
+            make_document(kvp("result", make_document(kvp("bulkWriteResult", types::b_null{}))));
+        return doc;
+    }
+}
+
+document::value delete_key(entity::map& map, const std::string& object, document::view operation) {
+    auto arguments = operation["arguments"].get_document().value;
+    auto key = arguments["id"].get_value();
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.delete_key(key);
+
+    auto doc =
+        make_document(kvp("result", make_document(kvp("deletedCount", result.deleted_count()))));
+    return doc;
+}
+
+document::value get_key(entity::map& map, const std::string& object, document::view operation) {
+    auto arguments = operation["arguments"].get_document().value;
+    auto key = arguments["id"].get_value();
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.get_key(key);
+
+    if (result) {
+        auto doc = make_document(kvp("result", *result));
+        return doc;
+    } else {
+        auto doc = make_document(kvp("result", types::b_null{}));
+        return doc;
+    }
+}
+
+document::value get_keys(entity::map& map, const std::string& object, document::view operation) {
+    (void)operation;
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto cursor = client_encryption.get_keys();
+
+    return make_document(kvp("result", [&cursor](builder::basic::sub_array array) {
+        for (auto&& document : cursor) {
+            array.append(document);
+        }
+    }));
+}
+
+document::value create_data_key(entity::map& map,
+                                const std::string& object,
+                                document::view operation) {
+    std::string provider;
+    if (operation["arguments"]["kmsProvider"]) {
+        provider = std::string(operation["arguments"]["kmsProvider"].get_string().value);
+    }
+
+    options::data_key data_key_opts;
+    if (operation["arguments"]["opts"]["masterKey"]) {
+        auto master_key = operation["arguments"]["opts"]["masterKey"].get_document().value;
+        data_key_opts.master_key(master_key);
+    }
+    if (operation["arguments"]["opts"]["keyAltNames"]) {
+        std::vector<std::string> key_alt_names;
+        auto arr = operation["arguments"]["opts"]["keyAltNames"].get_array().value;
+        for (const auto& it : arr) {
+            key_alt_names.push_back(std::string(it.get_string().value));
+        }
+        data_key_opts.key_alt_names(key_alt_names);
+    }
+    if (operation["arguments"]["opts"]["keyMaterial"]) {
+        auto bin = operation["arguments"]["opts"]["keyMaterial"].get_binary();
+        std::vector<uint8_t> bytes(bin.bytes, bin.bytes + bin.size);
+        data_key_opts.key_material(bytes);
+    }
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.create_data_key(provider, data_key_opts);
+
+    return make_document(kvp("result", result));
+}
+
+document::value add_key_alt_name(entity::map& map,
+                                 const std::string& object,
+                                 document::view operation) {
+    auto arguments = operation["arguments"].get_document().value;
+    auto id = arguments["id"].get_value();
+    std::string key_alt_name = std::string(arguments["keyAltName"].get_string().value);
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.add_key_alt_name(id, key_alt_name);
+
+    if (result) {
+        auto doc = make_document(kvp("result", *result));
+        return doc;
+    } else {
+        auto doc = make_document(kvp("result", types::b_null{}));
+        return doc;
+    }
+}
+
+document::value get_key_by_alt_name(entity::map& map,
+                                    const std::string& object,
+                                    document::view operation) {
+    auto arguments = operation["arguments"].get_document().value;
+    std::string key_alt_name = std::string(arguments["keyAltName"].get_string().value);
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.get_key_by_alt_name(key_alt_name);
+
+    if (result) {
+        auto doc = make_document(kvp("result", *result));
+        return doc;
+    } else {
+        auto doc = make_document(kvp("result", types::b_null{}));
+        return doc;
+    }
+}
+
+document::value remove_key_alt_name(entity::map& map,
+                                    const std::string& object,
+                                    document::view operation) {
+    auto arguments = operation["arguments"].get_document().value;
+    auto id = arguments["id"].get_value();
+    std::string key_alt_name = std::string(arguments["keyAltName"].get_string().value);
+
+    client_encryption& client_encryption = map.get_client_encryption(object);
+    auto result = client_encryption.remove_key_alt_name(id, key_alt_name);
+
+    if (result) {
+        auto doc = make_document(kvp("result", *result));
+        return doc;
+    } else {
+        auto doc = make_document(kvp("result", types::b_null{}));
+        return doc;
+    }
+}
+
 document::value create_find_cursor(entity::map& map,
                                    const std::string& object,
                                    client_session* session,
@@ -1639,8 +1883,14 @@ document::value operations::run(entity::map& entity_map,
     if (name == "listCollections") {
         return list_collections(entity_map, get_session(op_view, entity_map), object, op_view);
     }
+    if (name == "listCollectionNames") {
+        return list_collection_names(entity_map, get_session(op_view, entity_map), object, op_view);
+    }
     if (name == "listDatabases") {
-        return list_databases(entity_map, get_session(op_view, entity_map), object);
+        return list_databases(entity_map, get_session(op_view, entity_map), object, op_view);
+    }
+    if (name == "listDatabaseNames") {
+        return list_database_names(entity_map, get_session(op_view, entity_map), object, op_view);
     }
     if (name == "listIndexes") {
         return list_indexes(entity_map, get_session(op_view, entity_map), object);
@@ -1855,6 +2105,30 @@ document::value operations::run(entity::map& entity_map,
         auto& coll = entity_map.get_collection(object);
         return distinct(coll, get_session(op_view, entity_map), op_view);
     }
+    if (name == "rewrapManyDataKey") {
+        return rewrap_many_datakey(entity_map, object, op_view);
+    }
+    if (name == "deleteKey") {
+        return delete_key(entity_map, object, op_view);
+    }
+    if (name == "getKey") {
+        return get_key(entity_map, object, op_view);
+    }
+    if (name == "getKeys") {
+        return get_keys(entity_map, object, op_view);
+    }
+    if (name == "createDataKey") {
+        return create_data_key(entity_map, object, op_view);
+    }
+    if (name == "addKeyAltName") {
+        return add_key_alt_name(entity_map, object, op_view);
+    }
+    if (name == "getKeyByAltName") {
+        return get_key_by_alt_name(entity_map, object, op_view);
+    }
+    if (name == "removeKeyAltName") {
+        return remove_key_alt_name(entity_map, object, op_view);
+    }
 
     throw std::logic_error{"unsupported operation: " + name};
 }
